@@ -446,9 +446,9 @@ SSqlException SODBC::sPerrorException( const std::string & reason )
   return SSqlException( reason );
 }
 
-SSqlStatement* SODBC::prepare(const string& query, int nparams)
+std::unique_ptr<SSqlStatement> SODBC::prepare(const string& query, int nparams)
 {
-  return new SODBCStatement(query, m_log, nparams, m_connection);
+  return std::unique_ptr<SSqlStatement>(new SODBCStatement(query, m_log, nparams, m_connection));
 }
 
 
