@@ -1068,6 +1068,7 @@ DNSPacket *PacketHandler::doQuestion(DNSPacket *p)
   vector<DNSZoneRecord> rrset;
   bool weDone=0, weRedirected=0, weHaveUnauth=0;
   DNSName haveAlias;
+  uint8_t aliasScopeMask;
 
   DNSPacket *r=0;
   bool noCache=false;
@@ -1288,6 +1289,7 @@ DNSPacket *PacketHandler::doQuestion(DNSPacket *p)
     B.lookup(QType(QType::ANY), target, p, sd.domain_id);
     rrset.clear();
     haveAlias.trimToLabels(0);
+    aliasScopeMask = 0;
     weDone = weRedirected = weHaveUnauth =  false;
     
     while(B.get(rr)) {
@@ -1313,6 +1315,7 @@ DNSPacket *PacketHandler::doQuestion(DNSPacket *p)
           continue;
         }
         haveAlias=getRR<ALIASRecordContent>(rr.dr)->d_content;
+        aliasScopeMask=rr.scopeMask;
       }
 
       // Filter out all SOA's and add them in later
@@ -1343,7 +1346,7 @@ DNSPacket *PacketHandler::doQuestion(DNSPacket *p)
 
     if(!haveAlias.empty() && (!weDone || p->qtype.getCode() == QType::ANY)) {
       DLOG(L<<Logger::Warning<<"Found nothing that matched for '"<<target<<"', but did get alias to '"<<haveAlias<<"', referring"<<endl);
-      DP->completePacket(r, haveAlias, target);
+      DP->completePacket(r, haveAlias, target, aliasScopeMask);
       return 0;
     }
 
